# -*- coding: utf-8 -*-

import tarfile

import pytest
import yaml

pytestmark = pytest.mark.integration
pytest_plugins = 'pytester'


def create_role(
        role_base_path, role_name, test_playbook=None, dependencies=None,
        inventory=None):
    if not dependencies:
        dependencies = []
    if not inventory:
        inventory = '127.0.0.1 ansible_connection=local'

    # ensure some directory other than cwd is used
    # as ansible v2 seems to automatically look for roles in cwd
    role_path = role_base_path.join(role_name)

    # create role meta
    meta_info = {
        'galaxy_info': {
            'author': 'John Doe'
        },
        'dependencies': ansible_role_dependencies(dependencies),
    }
    role_path.join('meta', 'main.yml').write(yaml.dump(meta_info), ensure=True)

    # create role tasks
    role_path.join('tasks', 'main.yml').write('''---
- file:
    path: "{0!s}"
    state: touch
'''.format(role_path.join('.run')), ensure=True)

    if test_playbook:
        # create role tests
        role_path.join('tests', 'inventory').write(inventory, ensure=True)
        role_path.join('tests', 'test_playbook.yml').write(
            test_playbook, ensure=True)

    archive_path = role_base_path.join(role_name + '.tar.gz')
    with tarfile.open(str(archive_path), 'w:gz') as tar:
        tar.add(str(role_path), arcname=role_path.basename)

    return str(archive_path)


def ansible_role_dependencies(dependencies):
    return [dict(name=dependency.split('/')[-1][:-7], src=dependency)
            for dependency in dependencies]


def run(testdir):
    return testdir.inline_run('-s')


def test_passed_on_selfcontained_role(testdir):
    local_role_base_path = testdir.tmpdir.join('local-role-base')

    create_role(local_role_base_path, 'role1', '''---
- hosts: 127.0.0.1
  roles:
    - role: role1

- hosts: 127.0.0.1
  tasks:
    - name: assert role1 run
      file:
        path: "{0!s}"
        state: file
      tags: test
'''.format(local_role_base_path.join('role1', '.run')))

    result = run(testdir)
    result.assertoutcome(passed=1)


def test_failed_on_selfcontained_role_without_meta(testdir):
    local_role_base_path = testdir.tmpdir.join('local-role-base')

    create_role(local_role_base_path, 'role1', '''---
- hosts: 127.0.0.1
  roles:
    - role: role1

- hosts: 127.0.0.1
  tasks:
    - name: assert role1 run
      file:
        path: "{0!s}"
        state: file
      tags: test
'''.format(local_role_base_path.join('role1', '.run')))

    local_role_base_path.join('role1', 'meta').remove(rec=1)

    result = run(testdir)
    result.assertoutcome(failed=1)


def test_passed_on_role_with_dependent_role_beside(testdir):
    local_role_base_path = testdir.tmpdir.join('local-role-base')

    role1_url = create_role(local_role_base_path, 'role1')

    create_role(local_role_base_path, 'role2', '''---
- hosts: 127.0.0.1
  roles:
    - role: role2

- hosts: 127.0.0.1
  tasks:
    - name: assert role1 run
      file:
        path: "{0!s}"
        state: file
      tags: test

    - name: assert role2 run
      file:
        path: "{1!s}"
        state: file
      tags: test
'''.format(
        local_role_base_path.join('role1', '.run'),
        local_role_base_path.join('role2', '.run')
    ), dependencies=[role1_url])

    result = run(testdir)
    result.assertoutcome(passed=2)


def test_passed_on_role_with_multi_level_dependent_role_beside(testdir):
    local_role_base_path = testdir.tmpdir.join('local-role-base')

    role1_url = create_role(
        local_role_base_path, 'role1')

    role2_url = create_role(
        local_role_base_path, 'role2', dependencies=[role1_url])

    create_role(local_role_base_path, 'role3', '''---
- hosts: 127.0.0.1
  roles:
    - role: role3

- hosts: 127.0.0.1
  tasks:
    - name: assert role1 run
      file:
        path: "{0!s}"
        state: file
      tags: test

    - name: assert role2 run
      file:
        path: "{1!s}"
        state: file
      tags: test

    - name: assert role3 run
      file:
        path: "{2!s}"
        state: file
      tags: test
'''.format(
        local_role_base_path.join('role1', '.run'),
        local_role_base_path.join('role2', '.run'),
        local_role_base_path.join('role3', '.run')
    ), dependencies=[role2_url])

    result = run(testdir)
    result.assertoutcome(passed=3)


def test_passed_on_role_with_external_dependent_role(testdir):
    external_role_base_path = testdir.tmpdir.join('external-role-base')
    local_role_base_path = testdir.tmpdir.join('local-role-base')

    role1_url = create_role(external_role_base_path, 'role1')

    create_role(local_role_base_path, 'role2', '''---
- hosts: 127.0.0.1
  roles:
    - role: role2

- hosts: 127.0.0.1
  tasks:
    - name: assert role1 run
      file:
        path: "{0!s}"
        state: file
      tags: test

    - name: assert role2 run
      file:
        path: "{1!s}"
        state: file
      tags: test
'''.format(
        external_role_base_path.join('role1', '.run'),
        local_role_base_path.join('role2', '.run')
    ), dependencies=[role1_url])

    result = run(testdir)
    result.assertoutcome(passed=2)


def test_passed_on_role_with_multi_level_external_dependent_role(testdir):
    external_role_base_path = testdir.tmpdir.join('external-role-base')
    local_role_base_path = testdir.tmpdir.join('local-role-base')

    role1_url = create_role(
        external_role_base_path, 'role1')

    role2_url = create_role(
        external_role_base_path, 'role2', dependencies=[role1_url])

    create_role(local_role_base_path, 'role3', '''---
- hosts: 127.0.0.1
  roles:
    - role: role3

- hosts: 127.0.0.1
  tasks:
    - name: assert role1 run
      file:
        path: "{0!s}"
        state: file
      tags: test

    - name: assert role2 run
      file:
        path: "{1!s}"
        state: file
      tags: test

    - name: assert role3 run
      file:
        path: "{2!s}"
        state: file
      tags: test
'''.format(
        external_role_base_path.join('role1', '.run'),
        external_role_base_path.join('role2', '.run'),
        local_role_base_path.join('role3', '.run')
    ), dependencies=[role2_url])

    result = run(testdir)
    result.assertoutcome(passed=3)


def test_passed_on_role_with_external_soft_dependent_role(testdir):
    external_role_base_path = testdir.tmpdir.join('external-role-base')
    local_role_base_path = testdir.tmpdir.join('local-role-base')

    role1_url = create_role(external_role_base_path, 'role1')

    create_role(local_role_base_path, 'role2', '''---
- hosts: 127.0.0.1
  roles:
    - role: role1
    - role: role2

- hosts: 127.0.0.1
  tasks:
    - name: assert role1 run
      file:
        path: "{0!s}"
        state: file
      tags: test

    - name: assert role2 run
      file:
        path: "{1!s}"
        state: file
      tags: test
'''.format(
        external_role_base_path.join('role1', '.run'),
        local_role_base_path.join('role2', '.run')
    ), dependencies=[])

    local_role_base_path.join('role2', 'tests', 'requirements.yml').write('''---
- name: role1
  src: "{0!s}"
'''.format(role1_url), ensure=True)

    result = run(testdir)
    result.assertoutcome(passed=2)


def test_passed_on_role_with_multi_level_external_soft_dependent_role(testdir):
    external_role_base_path = testdir.tmpdir.join('external-role-base')
    local_role_base_path = testdir.tmpdir.join('local-role-base')

    role1_url = create_role(
        external_role_base_path, 'role1')

    role2_url = create_role(
        external_role_base_path, 'role2', dependencies=[role1_url])

    create_role(local_role_base_path, 'role3', '''---
- hosts: 127.0.0.1
  roles:
    - role: role2
    - role: role3

- hosts: 127.0.0.1
  tasks:
    - name: assert role1 run
      file:
        path: "{0!s}"
        state: file
      tags: test

    - name: assert role2 run
      file:
        path: "{1!s}"
        state: file
      tags: test

    - name: assert role3 run
      file:
        path: "{2!s}"
        state: file
      tags: test
'''.format(
        external_role_base_path.join('role1', '.run'),
        external_role_base_path.join('role2', '.run'),
        local_role_base_path.join('role3', '.run')
    ), dependencies=[])

    local_role_base_path.join('role3', 'tests', 'requirements.yml').write('''---
- name: role2
  src: "{0!s}"
'''.format(role2_url), ensure=True)

    result = run(testdir)
    result.assertoutcome(passed=3)


def test_dependency_beside_takes_precedence_over_soft_dependency(testdir):
    external_role_base_path = testdir.tmpdir.join('external-role-base')
    local_role_base_path = testdir.tmpdir.join('local-role-base')

    create_role(local_role_base_path, 'role1')
    role1soft_url = create_role(external_role_base_path, 'role1soft')

    create_role(local_role_base_path, 'role2', '''---
- hosts: 127.0.0.1
  roles:
    - role: role1
    - role: role2

- hosts: 127.0.0.1
  tasks:
    - name: assert role1 run
      file:
        path: "{0!s}"
        state: file
      tags: test

    - name: assert role1soft not run
      file:
        path: "{1!s}"
        state: absent
      tags: test

    - name: assert role2 run
      file:
        path: "{2!s}"
        state: file
      tags: test
'''.format(
        local_role_base_path.join('role1', '.run'),
        external_role_base_path.join('role1soft', '.run'),
        local_role_base_path.join('role2', '.run')
    ), dependencies=[])

    local_role_base_path.join('role2', 'tests', 'requirements.yml').write('''---
- name: role1
  src: "{0!s}"
'''.format(role1soft_url), ensure=True)

    result = run(testdir)
    result.assertoutcome(passed=3)


def test_dependency_beside_takes_precedence_over_role_dependency(testdir):
    external_role_base_path = testdir.tmpdir.join('external-role-base')
    local_role_base_path = testdir.tmpdir.join('local-role-base')

    create_role(local_role_base_path, 'role1')
    role1external_url = create_role(external_role_base_path, 'role1')

    create_role(local_role_base_path, 'role2', '''---
- hosts: 127.0.0.1
  roles:
    - role: role2

- hosts: 127.0.0.1
  tasks:
    - name: assert role1 run
      file:
        path: "{0!s}"
        state: file
      tags: test

    - name: assert role1external not run
      file:
        path: "{1!s}"
        state: absent
      tags: test

    - name: assert role2 run
      file:
        path: "{2!s}"
        state: file
      tags: test
'''.format(
        local_role_base_path.join('role1', '.run'),
        external_role_base_path.join('role1', '.run'),
        local_role_base_path.join('role2', '.run')
    ), dependencies=[role1external_url])

    result = run(testdir)
    result.assertoutcome(passed=3)


def test_soft_dependency_takes_precedence_over_role_dependency(testdir):
    external_role_base_path = testdir.tmpdir.join('external-role-base')
    local_role_base_path = testdir.tmpdir.join('local-role-base')

    role1_url = create_role(external_role_base_path, 'role1')
    role1soft_url = create_role(external_role_base_path, 'role1soft')

    create_role(local_role_base_path, 'role2', '''---
- hosts: 127.0.0.1
  roles:
    - role: role1
    - role: role2

- hosts: 127.0.0.1
  tasks:
    - name: assert role1 not run
      file:
        path: "{0!s}"
        state: absent
      tags: test

    - name: assert role1soft run
      file:
        path: "{1!s}"
        state: file
      tags: test

    - name: assert role2 run
      file:
        path: "{2!s}"
        state: file
      tags: test
'''.format(
        external_role_base_path.join('role1', '.run'),
        external_role_base_path.join('role1soft', '.run'),
        local_role_base_path.join('role2', '.run')
    ), dependencies=[role1_url])

    local_role_base_path.join('role2', 'tests', 'requirements.yml').write('''---
- name: role1
  src: "{0!s}"
'''.format(role1soft_url), ensure=True)

    result = run(testdir)
    result.assertoutcome(passed=3)


def test_failed_on_unresolvable_role(testdir):
    local_role_base_path = testdir.tmpdir.join('local-role-base')

    create_role(local_role_base_path, 'role1', '''---
- hosts: 127.0.0.1
  roles:
    - role: role2

- hosts: 127.0.0.1
  tasks:
    - name: task1
      ping:
      tags: test
''')

    result = run(testdir)
    result.assertoutcome(failed=1)



# -*- coding: utf-8 -*-

import pytest

pytestmark = pytest.mark.integration
pytest_plugins = 'pytester'


def test_junitxml_removes_classname_yml_extension(testdir):
    testdir.makefile('', inventory='127.0.0.1 ansible_connection=local')
    testdir.makefile('.yml', test_playbook='''---
- hosts: 127.0.0.1
  tasks:
    - name: host is reachable
      ping:
      tags: test
''')

    junit_xml_path = testdir.tmpdir.join('junit.xml')

    testdir.inline_run('--junitxml=' + str(junit_xml_path))

    junit_xml_content = junit_xml_path.read()
    assert 'classname="test_playbook"' in junit_xml_content


def test_junitxml_does_not_remove_task_name_with_yml_extension(testdir):
    testdir.makefile('', inventory='127.0.0.1 ansible_connection=local')
    testdir.makefile('.yml', test_playbook='''---
- hosts: 127.0.0.1
  tasks:
    - name: update appconfig.yml
      ping:
      tags: test
''')

    junit_xml_path = testdir.tmpdir.join('junit.xml')

    testdir.inline_run('--junitxml=' + str(junit_xml_path))

    junit_xml_content = junit_xml_path.read()
    assert 'name="update appconfig.yml"' in junit_xml_content



# -*- coding: utf-8 -*-

import docker
import docker.errors
import docker.utils
import pytest
import yaml

pytestmark = pytest.mark.integration
pytest_plugins = 'pytester'


@pytest.fixture
def docker_client(mocker):
    proxy_method_names = (
        'create_container',
        'inspect_image',
        'pull',
        'remove_container',
        'start',
    )

    client = docker.Client(
        version='auto',
        **docker.utils.kwargs_from_env(assert_hostname=False))

    for proxy_method_name in proxy_method_names:
        origin_method = getattr(client, proxy_method_name)
        mocker.patch.object(client, proxy_method_name).side_effect = origin_method

    mocker.patch('docker.Client').return_value = client

    return client


@pytest.fixture
def docker_client_unavailable(mocker):
    client = docker.Client(base_url='unix://tmp/docker.unavailable.sock')

    mocker.patch('docker.Client').return_value = client

    return client


def create_playbook_and_run(testdir, playbook, inventory=None):
    testdir.makefile('.yml', test_playbook=playbook)

    if inventory:
        testdir.makefile('', inventory=inventory)

    return testdir.inline_run('-s')


def test_docker_is_not_initialized_when_not_used(testdir, mocker):
    mock_docker_client = mocker.patch('docker.Client', autospec=True)

    result = create_playbook_and_run(testdir, '''---
- hosts: default
  tasks:
    - name: host is reachable
      ping:
      tags: test
''', inventory='default ansible_connection=local')

    result.assertoutcome(passed=1)

    assert not mock_docker_client.called


def test_docker_is_not_initialized_when_collect_only(testdir, mocker):
    mock_docker_client = mocker.patch('docker.Client', autospec=True)

    testdir.makefile(
        '', inventory='guesthostname goodplay_image=busybox:latest')
    testdir.makefile('.yml', test_playbook='''---
- hosts: guesthostname
  gather_facts: no

  tasks:
    - name: task 1
      raw: "ls -la /"
      changed_when: False
      tags: test
''')

    items, result = testdir.inline_genitems()

    result.assertoutcome()
    assert len(items) == 1
    assert items[0].name == 'task 1'

    assert not mock_docker_client.called


def test_docker_used_and_not_available(
        testdir, docker_client_unavailable):
    result = create_playbook_and_run(testdir, '''---
- hosts: guesthostname
  gather_facts: no

  tasks:
    - name: task 1
      raw: "ls -la /"
      changed_when: False
      tags: test
''', inventory='guesthostname goodplay_image=busybox:latest')

    result.assertoutcome(failed=1)


def test_docker_pull_called_when_image_does_not_exist(
        testdir, docker_client):
    docker_client.inspect_image.side_effect = \
        docker.errors.NotFound('image not found', None,
                               explanation='simulate missing docker image')

    result = create_playbook_and_run(testdir, '''---
- hosts: guesthostname
  gather_facts: no

  tasks:
    - name: task 1
      raw: "ls -la /"
      changed_when: False
      tags: test
''', inventory='guesthostname goodplay_image=busybox:latest')

    result.assertoutcome(passed=1)

    docker_client.inspect_image.assert_called_once_with('busybox:latest')
    docker_client.pull.assert_called_once_with('busybox:latest')


def test_docker_pull_is_called_once_per_image_when_multiple_times(
        testdir, docker_client):
    docker_client.inspect_image.side_effect = \
        docker.errors.NotFound('image not found', None,
                               explanation='simulate missing docker image')

    result = create_playbook_and_run(testdir, '''---
- hosts: guesthostname1:guesthostname2
  gather_facts: no

  tasks:
    - name: task 1
      raw: "ls -la /"
      changed_when: False
      tags: test
''', inventory='''guesthostname1 goodplay_image=busybox:latest
guesthostname2 goodplay_image=busybox:latest
''')

    result.assertoutcome(passed=1)

    docker_client.inspect_image.assert_called_once_with('busybox:latest')
    docker_client.pull.assert_called_once_with('busybox:latest')


def test_docker_pull_not_called_when_image_already_exists(
        testdir, docker_client):
    # ensure image is present
    docker_client.pull('busybox:latest')
    docker_client.pull.reset_mock()

    result = create_playbook_and_run(testdir, '''---
- hosts: guesthostname
  gather_facts: no

  tasks:
    - name: task 1
      raw: "ls -la /"
      changed_when: False
      tags: test
''', inventory='guesthostname goodplay_image=busybox:latest')

    result.assertoutcome(passed=1)

    docker_client.inspect_image.assert_called_once_with('busybox:latest')
    docker_client.pull.assert_not_called()


def test_failed_when_goodplay_platform_cannot_be_resolved(
        testdir, docker_client):
    result = create_playbook_and_run(testdir, '''---
- hosts: guesthostname
  gather_facts: no

  tasks:
    - name: task 1
      raw: "ls -la /"
      changed_when: False
      tags: test
''', inventory='guesthostname goodplay_platform=thename:theversion')

    result.assertoutcome(failed=1)

    message = "goodplay_platform 'thename:theversion' specified in inventory for host " \
        "'guesthostname' not found in .goodplay.yml"
    assert message in str(result.getfailures()[0].longrepr)


def test_docker_pull_called_with_resolved_goodplay_platform(
        testdir, docker_client):
    docker_client.inspect_image.side_effect = \
        docker.errors.NotFound('image not found', None,
                               explanation='simulate missing docker image')

    testdir.tmpdir.join('.goodplay.yml').write('''---
platforms:
  - name: thename
    version: theversion
    image: busybox:latest
''', ensure=True)

    result = create_playbook_and_run(testdir, '''---
- hosts: guesthostname
  gather_facts: no

  tasks:
    - name: task 1
      raw: "ls -la /"
      changed_when: False
      tags: test
''', inventory='guesthostname goodplay_platform=thename:theversion')

    docker_client.inspect_image.assert_called_once_with('busybox:latest')
    docker_client.pull.assert_called_once_with('busybox:latest')

    result.assertoutcome(passed=1)


def test_started_docker_container_is_removed_after_successful_run(
        testdir, docker_client):
    result = create_playbook_and_run(testdir, '''---
- hosts: guesthostname
  gather_facts: no

  tasks:
    - name: task 1
      raw: "ls -la /"
      changed_when: False
      tags: test
''', inventory='guesthostname goodplay_image=busybox:latest')

    result.assertoutcome(passed=1)

    assert docker_client.start.call_count == 1

    started_container_id = docker_client.start.call_args[0][0]
    docker_client.remove_container.assert_called_once_with(
        started_container_id, force=True)


def test_started_docker_container_is_removed_after_failed_run(
        testdir, docker_client):
    result = create_playbook_and_run(testdir, '''---
- hosts: guesthostname
  gather_facts: no

  tasks:
    - name: intentionally failing task
      raw: who
      failed_when: True
      tags: test
''', inventory='guesthostname goodplay_image=busybox:latest')

    result.assertoutcome(failed=1)

    assert docker_client.start.call_count == 1

    started_container_id = docker_client.start.call_args[0][0]
    docker_client.remove_container.assert_called_once_with(
        started_container_id, force=True)


def test_group_vars_directory_beside_inventory_file_is_incorporated(testdir):
    group_vars_path = testdir.tmpdir.mkdir('group_vars')
    group_vars_path.join('all').write('hello: world')

    result = create_playbook_and_run(testdir, '''---
- hosts: guesthostname
  gather_facts: no

  tasks:
    - name: task 1
      raw: 'echo -n {{ hello }}'
      register: echo_result
      failed_when: echo_result.stdout != 'world'
      changed_when: False
      tags: test
''', inventory='guesthostname goodplay_image=busybox:latest')

    result.assertoutcome(passed=1)


def test_group_vars_directory_beside_inventory_directory_is_incorporated(
        testdir):
    inventory_path = testdir.tmpdir.mkdir('inventory')
    inventory_path.join('static').write(
        'guesthostname goodplay_image=busybox:latest')

    group_vars_path = inventory_path.mkdir('group_vars')
    group_vars_path.join('all').write('hello: world')

    result = create_playbook_and_run(testdir, '''---
- hosts: guesthostname
  gather_facts: no

  tasks:
    - name: task 1
      raw: 'echo -n {{ hello }}'
      register: echo_result
      failed_when: echo_result.stdout != 'world'
      changed_when: False
      tags: test
''')

    result.assertoutcome(passed=1)


def test_host_vars_directory_beside_inventory_file_is_incorporated(testdir):
    host_vars_path = testdir.tmpdir.mkdir('host_vars')
    host_vars_path.join('guesthostname').write('hello: world')

    result = create_playbook_and_run(testdir, '''---
- hosts: guesthostname
  gather_facts: no

  tasks:
    - name: task 1
      raw: 'echo -n {{ hello }}'
      register: echo_result
      failed_when: echo_result.stdout != 'world'
      changed_when: False
      tags: test
''', inventory='guesthostname goodplay_image=busybox:latest')

    result.assertoutcome(passed=1)


def test_host_vars_directory_beside_inventory_directory_is_incorporated(
        testdir):
    inventory_path = testdir.tmpdir.mkdir('inventory')
    inventory_path.join('static').write(
        'guesthostname goodplay_image=busybox:latest')

    host_vars_path = inventory_path.mkdir('host_vars')
    host_vars_path.join('guesthostname').write('hello: world')

    result = create_playbook_and_run(testdir, '''---
- hosts: guesthostname
  gather_facts: no

  tasks:
    - name: task 1
      raw: 'echo -n {{ hello }}'
      register: echo_result
      failed_when: echo_result.stdout != 'world'
      changed_when: False
      tags: test
''')

    result.assertoutcome(passed=1)


def test_role_with_goodplay_platform_wildcard(
        testdir, docker_client):
    local_role_base_path = testdir.tmpdir.join('local-role-base')

    # ensure some directory other than cwd is used
    # as ansible v2 seems to automatically look for roles in cwd
    role_path = local_role_base_path.join('role1')

    # create role meta
    meta_info = {
        'galaxy_info': {
            'author': 'John Doe',
            'platforms': [{
                'name': 'EL',
                'versions': [6, 7]
            }]
        },
        'dependencies': [],
    }
    role_path.join('meta', 'main.yml').write(yaml.dump(meta_info), ensure=True)

    # create role tasks
    role_path.join('tasks', 'main.yml').write('''---
- ping:
''', ensure=True)

    # create goodplay config
    role_path.join('tests', '.goodplay.yml').write('''---
platforms:
  - name: EL
    version: 6
    image: centos:centos6

  - name: EL
    version: 7
    image: centos:centos7
''', ensure=True)

    # create role tests
    role_path.join('tests', 'inventory').write(
        'default goodplay_platform=*', ensure=True)
    role_path.join('tests', 'test_playbook.yml').write('''---
- hosts: default
  tasks:
    - name: host is reachable
      ping:
      tags: test
''', ensure=True)

    result = testdir.inline_run('-s')

    result.assertoutcome(passed=2)

    assert docker_client.create_container.call_count == 2

    _, _, kwargs = docker_client.create_container.mock_calls[0]
    assert kwargs['image'] == 'centos:centos6'

    _, _, kwargs = docker_client.create_container.mock_calls[1]
    assert kwargs['image'] == 'centos:centos7'

    assert docker_client.start.call_count == 2



# -*- coding: utf-8 -*-

import pytest
import yaml

pytestmark = pytest.mark.integration
pytest_plugins = 'pytester'


@pytest.mark.integration
def test_nodeid_no_platform(testdir):
    testdir.makefile('', inventory='all')
    testdir.makefile('.yml', test_playbook='''---
- hosts: all
  tasks:
    - name: task1
      ping:
      tags: test
''')

    items, result = testdir.inline_genitems()

    assert len(items) == 1
    assert items[0].nodeid == 'test_playbook.yml::task1'

    result.assertoutcome()


@pytest.mark.integration
def test_nodeid_with_platform(testdir):
    role_path = testdir.tmpdir

    # create role meta
    meta_info = {
        'galaxy_info': {
            'author': 'John Doe',
            'platforms': [{
                'name': 'EL',
                'versions': [6, 7]
            }]
        },
        'dependencies': [],
    }
    role_path.join('meta', 'main.yml').write(yaml.dump(meta_info), ensure=True)

    # create role tasks
    role_path.join('tasks', 'main.yml').write('''---
- ping:
''', ensure=True)

    # create goodplay config
    role_path.join('tests', '.goodplay.yml').write('''---
platforms:
  - name: EL
    version: 6
    image: centos:centos6

  - name: EL
    version: 7
    image: centos:centos7
''', ensure=True)

    # create role tests
    role_path.join('tests', 'inventory').write(
        'default goodplay_platform=*', ensure=True)
    role_path.join('tests', 'test_playbook.yml').write('''---
- hosts: default
  tasks:
    - name: host is reachable
      ping:
      tags: test
''', ensure=True)

    items, result = testdir.inline_genitems()

    assert len(items) == 2
    assert items[0].nodeid == 'tests/test_playbook.yml::EL:6::host is reachable'
    assert items[1].nodeid == 'tests/test_playbook.yml::EL:7::host is reachable'

    result.assertoutcome()



# -*- coding: utf-8 -*-

import pytest

pytestmark = pytest.mark.integration
pytest_plugins = 'pytester'


def create_playbook_and_run(testdir, playbook, inventory=None):
    if not inventory:
        inventory = '127.0.0.1 ansible_connection=local'
    testdir.makefile('', inventory=inventory)
    testdir.makefile('.yml', test_playbook=playbook)

    return testdir.inline_run('-s')


def test_passed_on_non_changed_task(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: 127.0.0.1
  tasks:
    - name: task1
      ping:
      tags: test
''')

    result.assertoutcome(passed=1)


def test_passed_on_previously_changed_non_test_task(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: 127.0.0.1
  tasks:
    - name: intentionally changed task
      ping:
      changed_when: True

    - name: task1
      ping:
      tags: test
''')

    result.assertoutcome(passed=1)


def test_passed_on_previously_skipped_non_test_task(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: 127.0.0.1
  tasks:
    - name: intentionally skipped task
      ping:
      when: False

    - name: task1
      ping:
      tags: test
''')

    result.assertoutcome(passed=1)


def test_passed_on_previously_failed_non_test_task_with_ignore_errors(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: 127.0.0.1
  tasks:
    - name: intentionally failed task
      ping:
      failed_when: True
      ignore_errors: yes

    - name: task1
      ping:
      tags: test
''')

    result.assertoutcome(passed=1)


def test_passed_on_multiple_plays(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: 127.0.0.1
  tasks:
    - name: task1
      ping:
      tags: test

- hosts: 127.0.0.1
  tasks:
    - name: task2
      ping:
      tags: test
''')

    result.assertoutcome(passed=2)


def test_passed_without_gather_facts(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: 127.0.0.1
  gather_facts: no
  tasks:
    - name: task1
      ping:
      tags: test
''')

    result.assertoutcome(passed=1)


def test_skipped_outcome_takes_priority_over_passed(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: host1
  tasks:
    - name: avoid all test tasks skipped
      ping:
      tags: test

- hosts: host1:host2
  tasks:
    - name: task2
      ping:
      when: inventory_hostname != 'host2'
      tags: test
''', inventory='''host1 ansible_connection=local
host2 ansible_connection=local
''')

    result.assertoutcome(passed=1, skipped=1)


def test_skipped_on_previously_failed_non_test_task(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: 127.0.0.1
  tasks:
    - name: intentionally failed task
      ping:
      failed_when: True

    - name: task1
      ping:
      tags: test
''')

    result.assertoutcome(skipped=1, failed=1)


def test_skipped_multiple_on_previously_failed_non_test_task(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: 127.0.0.1
  tasks:
    - name: intentionally failed task
      ping:
      failed_when: True

    - name: task1
      ping:
      tags: test

    - name: task2
      ping:
      tags: test
''')

    result.assertoutcome(skipped=2, failed=1)


def test_failed_on_changed_task(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: 127.0.0.1
  tasks:
    - name: task1
      ping:
      changed_when: True
      tags: test
''')

    result.assertoutcome(failed=1)


def test_failed_on_failed_task(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: 127.0.0.1
  tasks:
    - name: task1
      ping:
      failed_when: True
      tags: test
''')

    result.assertoutcome(failed=1)


def test_failed_on_wait_for_timeout(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: 127.0.0.1
  tasks:
    - name: task1
      wait_for:
        path: /path/to/some/nonexisting/file
        timeout: 0
      tags: test
''')

    result.assertoutcome(failed=1)


def test_failed_on_unreachable_host_on_gather_facts(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: unreachable.host.local
  tasks:
    - name: task1
      ping:
      tags: test
''', inventory='unreachable.host.local')

    result.assertoutcome(skipped=1, failed=1)


def test_failed_on_single_failed_host(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: host1:host2
  tasks:
    - name: task1
      ping:
      failed_when: inventory_hostname == 'host1'
      tags: test
''', inventory='''host1 ansible_connection=local
host2 ansible_connection=local
''')

    result.assertoutcome(failed=1)


def test_failed_on_all_tasks_skipped(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: unknownhost
  tasks:
    - name: task1
      ping:
      tags: test
''')

    result.assertoutcome(skipped=1, failed=1)
    assert 'Failed: all test tasks have been skipped' in \
        str(result.getfailures()[0].longrepr)


def test_failed_outcome_takes_highest_priority(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: host1:host2:host3
  tasks:
    - name: task1
      ping:
      when: inventory_hostname != 'host2'
      failed_when: inventory_hostname == 'host3'
      tags: test
''', inventory='''host1 ansible_connection=local
host2 ansible_connection=local
host3 ansible_connection=local
''')

    result.assertoutcome(failed=1)


def test_failed_on_failing_non_test_task_after_passed_test_task(testdir):
    result = create_playbook_and_run(testdir, '''---
- hosts: 127.0.0.1
  tasks:
    - name: task1
      ping:
      tags: test

    - name: intentionally failed task
      ping:
      failed_when: True
''')

    result.assertoutcome(passed=1, failed=1)


def test_ansible_stdout_is_fully_consumed(testdir, capsys):
    create_playbook_and_run(testdir, '''---
- hosts: 127.0.0.1
  tasks:
    - name: task1
      ping:
      tags: test

    - name: intentionally failed task
      ping:
      failed_when: True
''')

    out, _ = capsys.readouterr()
    ansible_play_recap_part = 'failed=1'
    assert ansible_play_recap_part in out



# -*- coding: utf-8 -*-

import subprocess


def test_goodplay_cli_args_are_forwarded_to_pytest(tmpdir):
    tmpdir.join('inventory').write(
        '127.0.0.1 ansible_connection=local', ensure=True)

    tmpdir.join('test_playbook.yml').write('''---
- hosts: 127.0.0.1
  tasks:
    - name: task1
      ping:
      tags: test
''', ensure=True)

    stdout = subprocess.check_output(
        ['goodplay', '--collect-only'], cwd=str(tmpdir))
    expected = "<GoodplayTest 'task1'>"

    assert expected in stdout


def test_goodplay_cli_does_not_collect_python_tests(tmpdir):
    tmpdir.join('test_somepython.py').write('''
def test_something_python_related():
    pass
''', ensure=True)

    # pytest's exit code when no tests have been collected
    EXIT_NOTESTSCOLLECTED = 5

    goodplay_returncode = subprocess.call(
        ['goodplay'], cwd=str(tmpdir))

    assert goodplay_returncode == EXIT_NOTESTSCOLLECTED


def test_goodplay_cli_recursively_collects_from_subdirectories(tmpdir):
    tmpdir.join('some', 'subdir', 'inventory').write(
        '127.0.0.1 ansible_connection=local', ensure=True)

    tmpdir.join('some', 'subdir', 'test_playbook.yml').write('''---
- hosts: 127.0.0.1
  tasks:
    - name: task1
      ping:
      tags: test
''', ensure=True)

    stdout = subprocess.check_output(
        ['goodplay', '--collect-only'], cwd=str(tmpdir))
    expected = "<GoodplayTest 'task1'>"

    assert expected in stdout


def test_goodplay_cli_forwards_pytest_returncode(tmpdir):
    # pytest's exit code when no tests have been collected
    EXIT_NOTESTSCOLLECTED = 5

    goodplay_returncode = subprocess.call(
        ['goodplay'], cwd=str(tmpdir))

    assert goodplay_returncode == EXIT_NOTESTSCOLLECTED



# -*- coding: utf-8 -*-

import _pytest.junitxml


def patch_mangle_testnames():
    # monkeypatch mangle_testnames to remove .yml extension from playbook "class"
    mangle_testnames_orig = _pytest.junitxml.mangle_testnames

    def mangle_testnames_patch(names):
        names = mangle_testnames_orig(names)
        return [x[:-4] if x.endswith('.yml') else x for x in names[:-1]] + [names[-1]]
    _pytest.junitxml.mangle_testnames = mangle_testnames_patch
